<?php
namespace TYPO3\CMS\Extensionmanager\ViewHelpers\Be;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013 Francois Suter, <francois.suter@typo3.org>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *  A copy is found in the text file GPL.txt and important notices to the license
 *  from the author is found in LICENSE.txt distributed with these scripts.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use \TYPO3\CMS\Extensionmanager\Controller\AbstractController;

/**
 * Special view helper for the BE module of the Extension Manager. Loads JS code for triggering
 * refresh events.
 *
 * = Examples =
 *
 * <code title="Simple">
 * <em:be.container triggers="{triggers}" />
 * </code>
 * <output>
 * Writes some JS inline code
 * </output>
 */
class TriggerViewHelper extends \TYPO3\CMS\Fluid\ViewHelpers\Be\AbstractBackendViewHelper {

	/**
	 * Loads some JS inline code based on a list of triggers. This is used to reload the main
	 * menu when modules are loaded/unloaded.
	 *
	 * @param array $triggers Defined triggers to be forwarded to client (e.g. refreshing backend widgets)
	 * @return string This view helper does not return any content
	 * @see \TYPO3\CMS\Backend\Template\DocumentTemplate
	 * @see \TYPO3\CMS\Core\Page\PageRenderer
	 */
	public function render($triggers = array()) {
		$doc = $this->getDocInstance();
		$pageRenderer = $doc->getPageRenderer();
		// Handle triggers
		if (!empty($triggers[AbstractController::TRIGGER_RefreshModuleMenu])) {
			$pageRenderer->addJsInlineCode(
				AbstractController::TRIGGER_RefreshModuleMenu,
				'if (top.TYPO3ModuleMenu.refreshMenu) { top.TYPO3ModuleMenu.refreshMenu(); }'
			);
		}
		return '';
	}

}
